Package: TADCompare
Title: TADCompare: Identification and characterization of differential
        TADs
Version: 1.23.0
Authors@R: c(
    person(given = "Mikhail",
           family = "Dozmorov",
           role = c("aut", "cre"),
           email = "mikhail.dozmorov@gmail.com",
           comment = c(ORCID = "0000-0002-0086-8358")),
    person(given = "Kellen",
           family = "Cresswell",
           role = c("aut"))
           )
Description: 
    TADCompare is an R package designed to identify and characterize differential Topologically Associated 
    Domains (TADs) between multiple Hi-C contact matrices. It contains 
    functions for finding differential TADs between two datasets, finding
    differential TADs over time and identifying consensus TADs across multiple
    matrices. It takes all of the main types of HiC input and returns simple,
    comprehensive, easy to analyze results.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Imports: dplyr, PRIMME, cluster, Matrix, magrittr, HiCcompare, ggplot2,
        tidyr, ggpubr, RColorBrewer, reshape2, cowplot
Suggests: BiocStyle, knitr, rmarkdown, microbenchmark, testthat, covr,
        pheatmap, SpectralTAD, magick, qpdf
Depends: R (>= 4.0)
VignetteBuilder: knitr
biocViews: Software, HiC, Sequencing, FeatureExtraction, Clustering
BugReports: https://github.com/dozmorovlab/TADCompare/issues
URL: https://github.com/dozmorovlab/TADCompare
RoxygenNote: 7.3.1
git_url: https://git.bioconductor.org/packages/TADCompare
git_branch: devel
git_last_commit: bd28a95
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-18
NeedsCompilation: no
Packaged: 2026-05-18 23:38:43 UTC; biocbuild
Author: Mikhail Dozmorov [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0086-8358>),
  Kellen Cresswell [aut]
Maintainer: Mikhail Dozmorov <mikhail.dozmorov@gmail.com>
Built: R 4.6.0; ; 2026-05-19 05:11:49 UTC; unix
