# Generated by roxygen2: do not edit by hand

export(ConsensusTADs)
export(DiffPlot)
export(TADCompare)
export(TimeCompare)
import(Matrix)
import(PRIMME)
import(RColorBrewer)
import(cluster)
import(dplyr)
import(ggplot2)
import(magrittr)
importFrom(HiCcompare,sparse2full)
importFrom(cowplot,plot_grid)
importFrom(ggpubr,get_legend)
importFrom(reshape2,melt)
