#' Chromosome 22 time-varying contact matrices from Rao et al. 2017.
#'
#' Four 704x704 contact matrices representing 20, 40, 60, 180 minutes since
#' auxin treatment and removal from the HCT-116 cell line (50kb Resolution)
#'
#' @format A data frame with 704 rows and 704 variables:
#' @source \url{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE104334}
"time_mats"
