#' A subset of chomosome 2 contact matrix, GM12878 cell line. 
#'
#' A 1001x1001 contact matrix from the GM12878 cell line, chr2:8000000-48000000,
#' 40kb Resolution, data from Schmitt et al. 2016.
#'
#' @format A data frame with 1001 rows and 1001 variables:
#' @source \url{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE87112}
"GM12878.40kb.raw.chr2"
