% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GM12878.40kb.raw.chr2.R
\docType{data}
\name{GM12878.40kb.raw.chr2}
\alias{GM12878.40kb.raw.chr2}
\title{A subset of chomosome 2 contact matrix, GM12878 cell line.}
\format{
A data frame with 1001 rows and 1001 variables:
}
\source{
\url{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE87112}
}
\usage{
GM12878.40kb.raw.chr2
}
\description{
A 1001x1001 contact matrix from the GM12878 cell line, chr2:8000000-48000000,
40kb Resolution, data from Schmitt et al. 2016.
}
\keyword{datasets}
